@@ -93,13 +93,14 @@ func main() {
 	go signalHandler()
 
 	if conf.MonitoringAddress != "" {
-		launchMonitoringServer(conf)
+		go launchMonitoringServer(conf)
 	}
 
 	s := smtpsrv.NewServer()
 	s.Hostname = conf.Hostname
 	s.MaxDataSize = conf.MaxDataSizeMb * 1024 * 1024
 	s.HookPath = "hooks/"
+	s.HAProxyEnabled = conf.HaproxyIncoming
 
 	s.SetAliasesConfig(conf.SuffixSeparators, conf.DropCharacters)
 
@@ -159,7 +160,7 @@ func main() {
 	}
 	go stsCache.PeriodicallyRefresh(context.Background())
 
-	localC := &courier.Procmail{
+	localC := &courier.MDA{
 		Binary:  conf.MailDeliveryAgentBin,
 		Args:    conf.MailDeliveryAgentArgs,
 		Timeout: 30 * time.Second,
