@@ -8,7 +8,13 @@ init
 # Build the binary once, so we can use it and launch it in chamuyero scripts.
 # Otherwise, we not only spend time rebuilding it over and over, but also "go
 # run" masks the exit code, which is something we care about.
-go build dovecot-auth-cli.go
+if [ "${COVER_DIR}" != "" ]; then
+	go test -covermode=count -coverpkg=../../... -c \
+		$GOFLAGS -tags="coveragebin $GOTAGS"
+	cp coverage_wrapper dovecot-auth-cli
+else
+	go build $GOFLAGS -tags="$GOTAGS" dovecot-auth-cli.go
+fi
 
 for i in *.cmy; do
 	if ! chamuyero $i > $i.log 2>&1 ; then
@@ -19,3 +25,4 @@ for i in *.cmy; do
 done
 
 success
+exit 0
