@@ -8,6 +8,9 @@ import (
 
 	"blitiri.com.ar/go/chasquid/internal/testlib"
 	"blitiri.com.ar/go/log"
+	"github.com/google/go-cmp/cmp"
+	"google.golang.org/protobuf/proto"
+	"google.golang.org/protobuf/testing/protocmp"
 )
 
 func mustCreateConfig(t *testing.T, contents string) (string, string) {
@@ -24,32 +27,19 @@ func mustCreateConfig(t *testing.T, contents string) (string, string) {
 func TestEmptyConfig(t *testing.T) {
 	tmpDir, path := mustCreateConfig(t, "")
 	defer testlib.RemoveIfOk(t, tmpDir)
-	c, err := Load(path)
+	c, err := Load(path, "")
 	if err != nil {
 		t.Fatalf("error loading empty config: %v", err)
 	}
 
 	// Test the default values are set.
-
+	defaults := proto.Clone(defaultConfig).(*Config)
 	hostname, _ := os.Hostname()
-	if c.Hostname == "" || c.Hostname != hostname {
-		t.Errorf("invalid hostname %q, should be: %q", c.Hostname, hostname)
-	}
+	defaults.Hostname = hostname
 
-	if c.MaxDataSizeMb != 50 {
-		t.Errorf("max data size != 50: %d", c.MaxDataSizeMb)
-	}
-
-	if len(c.SmtpAddress) != 1 || c.SmtpAddress[0] != "systemd" {
-		t.Errorf("unexpected address default: %v", c.SmtpAddress)
-	}
-
-	if len(c.SubmissionAddress) != 1 || c.SubmissionAddress[0] != "systemd" {
-		t.Errorf("unexpected address default: %v", c.SubmissionAddress)
-	}
-
-	if c.MonitoringAddress != "" {
-		t.Errorf("monitoring address is set: %v", c.MonitoringAddress)
+	diff := cmp.Diff(defaults, c, protocmp.Transform())
+	if diff != "" {
+		t.Errorf("Load() mismatch (-want +got):\n%s", diff)
 	}
 
 	testLogConfig(c)
@@ -60,6 +50,8 @@ func TestFullConfig(t *testing.T) {
 		hostname: "joust"
 		smtp_address: ":1234"
 		smtp_address: ":5678"
+		submission_address: ":10001"
+		submission_address: ":10002"
 		monitoring_address: ":1111"
 		max_data_size_mb: 26
 	`
@@ -67,33 +59,49 @@ func TestFullConfig(t *testing.T) {
 	tmpDir, path := mustCreateConfig(t, confStr)
 	defer testlib.RemoveIfOk(t, tmpDir)
 
-	c, err := Load(path)
+	overrideStr := `
+		hostname: "proust"
+		submission_address: ":999"
+		dovecot_auth: true
+	`
+
+	expected := &Config{
+		Hostname:      "proust",
+		MaxDataSizeMb: 26,
+
+		SmtpAddress:              []string{":1234", ":5678"},
+		SubmissionAddress:        []string{":999"},
+		SubmissionOverTlsAddress: []string{"systemd"},
+		MonitoringAddress:        ":1111",
+
+		MailDeliveryAgentBin:  "maildrop",
+		MailDeliveryAgentArgs: []string{"-f", "%from%", "-d", "%to_user%"},
+
+		DataDir: "/var/lib/chasquid",
+
+		SuffixSeparators: "+",
+		DropCharacters:   ".",
+
+		MailLogPath: "<syslog>",
+
+		DovecotAuth: true,
+	}
+
+	c, err := Load(path, overrideStr)
 	if err != nil {
 		t.Fatalf("error loading non-existent config: %v", err)
 	}
 
-	if c.Hostname != "joust" {
-		t.Errorf("hostname %q != 'joust'", c.Hostname)
-	}
-
-	if c.MaxDataSizeMb != 26 {
-		t.Errorf("max data size != 26: %d", c.MaxDataSizeMb)
-	}
-
-	if len(c.SmtpAddress) != 2 ||
-		c.SmtpAddress[0] != ":1234" || c.SmtpAddress[1] != ":5678" {
-		t.Errorf("different address: %v", c.SmtpAddress)
-	}
-
-	if c.MonitoringAddress != ":1111" {
-		t.Errorf("monitoring address %q != ':1111;", c.MonitoringAddress)
+	diff := cmp.Diff(expected, c, protocmp.Transform())
+	if diff != "" {
+		t.Errorf("Load() mismatch (-want +got):\n%s", diff)
 	}
 
 	testLogConfig(c)
 }
 
 func TestErrorLoading(t *testing.T) {
-	c, err := Load("/does/not/exist")
+	c, err := Load("/does/not/exist", "")
 	if err == nil {
 		t.Fatalf("loaded a non-existent config: %v", c)
 	}
@@ -104,7 +112,18 @@ func TestBrokenConfig(t *testing.T) {
 		t, "<invalid> this is not a valid protobuf")
 	defer testlib.RemoveIfOk(t, tmpDir)
 
-	c, err := Load(path)
+	c, err := Load(path, "")
+	if err == nil {
+		t.Fatalf("loaded an invalid config: %v", c)
+	}
+}
+
+func TestBrokenOverride(t *testing.T) {
+	tmpDir, path := mustCreateConfig(
+		t, `hostname: "test"`)
+	defer testlib.RemoveIfOk(t, tmpDir)
+
+	c, err := Load(path, "broken override")
 	if err == nil {
 		t.Fatalf("loaded an invalid config: %v", c)
 	}
