@@ -5,6 +5,12 @@ set -e
 
 init
 
+mkdir -p .logs
+
+if ! chasquid --version > /dev/null; then
+	fail "chasquid --version failed"
+fi
+
 # This should fail, as it has no certificates.
 rm -f config/certs/testserver/*.pem
 if chasquid -v=2 --logfile=.logs/chasquid.log --config_dir=config; then
@@ -15,7 +21,6 @@ generate_certs_for testserver
 add_user user@testserver secretpassword
 add_user someone@testserver secretpassword
 
-mkdir -p .logs
 chasquid -v=2 --logfile=.logs/chasquid.log --config_dir=config &
 wait_until_ready 1025
 
